/*  
    BluePhone is a Bluetooth mobile phone manager
    Copyright (C) 2004 Ed Rayne <er241@cam.ac.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <glib.h>
#include <gtk/gtk.h>

#include "accessory.h"
#include "global.h"

enum 
{
	TITLE_COLUMN,
	COMMAND_COLUMN,
	N_COLUMNS
};

static void accessory_submenu (gint child_id, gint parent_id);
static void accessory_do_mouse ();

GtkTreeStore *menu_tree = NULL;	/* store the accessory menu structure here */
gint depth;			/* keep track of how deep into the menu system we are */
gint parentid;			/* keep track of which submenu we are in */
gint menu_items_count;

void
accessory_main_menu (void)
{
	GtkTreeIter parent, child, iter;
	gboolean valid;
	gchar *menu_str;

	menu_items_count = 0;
	
	if (!menu_tree)
	{
		/* no menu tree - better make one! */
		menu_tree = gtk_tree_store_new (N_COLUMNS,	/* number of columns */
				                G_TYPE_STRING,	/* menu item */
						G_TYPE_STRING);	/* command to execute */

		/* add rhythmbox options */
	
		gtk_tree_store_append (menu_tree, &parent, NULL);
		gtk_tree_store_set (menu_tree, &parent,
				    TITLE_COLUMN, "Rhythmbox",
				    COMMAND_COLUMN, NULL,
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Play / Pause",
				    COMMAND_COLUMN, "rhythmbox --play-pause",
				    -1);
	
		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Next",
				    COMMAND_COLUMN, "rhythmbox --next",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Previous",
				    COMMAND_COLUMN, "rhythmbox --previous",
				    -1);
	
		/* add totem options */

		gtk_tree_store_append (menu_tree, &parent, NULL);
		gtk_tree_store_set (menu_tree, &parent,
				    TITLE_COLUMN, "Totem",
				    COMMAND_COLUMN, NULL,
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Play / Pause",
				    COMMAND_COLUMN, "totem --play-pause",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Next",
				    COMMAND_COLUMN, "totem --next",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Previous",
				    COMMAND_COLUMN, "totem --previous",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Seek Forwards",
				    COMMAND_COLUMN, "totem --seek-fwd",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Seek Backwards",
				    COMMAND_COLUMN, "totem --seek-bwd",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Volume Up",
				    COMMAND_COLUMN, "totem --volume-up",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Volume Down",
				    COMMAND_COLUMN, "totem --volume-down",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Toggle Fullscreen",
				    COMMAND_COLUMN, "totem --fullscreen",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Quit",
				    COMMAND_COLUMN, "totem --quit",
				    -1);

	/* add xscreensaver options */

		gtk_tree_store_append (menu_tree, &parent, NULL);
		gtk_tree_store_set (menu_tree, &parent,
				    TITLE_COLUMN, "XScreensaver",
				    COMMAND_COLUMN, NULL,
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Activate",
				    COMMAND_COLUMN, "xscreensaver-command -activate",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Deactivate",
				    COMMAND_COLUMN, "xscreensaver-command -deactivate",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Next",
				    COMMAND_COLUMN, "xscreensaver-command -next",
				    -1);

		gtk_tree_store_append (menu_tree, &child, &parent);
		gtk_tree_store_set (menu_tree, &child,
				    TITLE_COLUMN, "Previous",
				    COMMAND_COLUMN, "xscreensaver-command -prev",
				    -1);
		
	}

	/*
	 * eventually this functionality will be moved once a GUI is 
	 * built for BluePhone. The menu structure will be stored in a very
	 * simple XML file that can be parsed easily using GLib and altered
	 * in the GUI to allow customisation
	 */

	menu_str = g_strdup ("");
	
	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (menu_tree), &iter);
	
	while (valid)
	{
		gchar *title;

		gtk_tree_model_get (GTK_TREE_MODEL (menu_tree), &iter,
				    TITLE_COLUMN, &title,
				    -1);

		menu_str = g_strdup_printf ("%s\"%s\",", menu_str, title);
		g_free (title);
		
		menu_items_count++;
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (menu_tree), &iter);
	}

	menu_str = g_strdup_printf ("AT*EASM=\"BluePhone\",0,1,%d,%s\"Mouse\"\r", (menu_items_count + 1), menu_str);
	g_print ("%s", menu_str);
	depth = 1;

	g_io_channel_write_chars (global.io_channel, menu_str, -1, NULL, NULL);
	g_io_channel_flush (global.io_channel, NULL);
}

void
accessory_menu_selected (gchar *msg)
{
	gint i, count = 0;
	gchar *tmp, *menu_str, *menu_title, *title;
	GtkTreeIter parent, child;
	gboolean valid;
	
	msg = msg + 7;
	
	i = atoi (msg);

	if (i == 0)
	{
		/* we've return a menu */
		if (depth > 1)
			accessory_main_menu ();
	}
	else
	{
		if (depth == 1)
		{
			if (i > menu_items_count)
			{
				/* mouse mode selected */
				accessory_do_mouse ();
			}
			else
			{
				/* submenu selected */
				parentid = i - 1;
				depth = 2;

				/* get an iterator for the parent node */
				gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (menu_tree), &parent, NULL, parentid);

				/* get the parent node title for the menu title */
				gtk_tree_model_get (GTK_TREE_MODEL (menu_tree), &parent,
						    TITLE_COLUMN, &title,
						    -1);
				menu_title = g_strdup_printf ("\"%s\"", title);
				g_free (title);
			
				/* walk through the child nodes creating the submenu */
				menu_str = g_strdup ("");

				valid = gtk_tree_model_iter_children (GTK_TREE_MODEL (menu_tree), &child, &parent);
				while (valid)
				{
					gtk_tree_model_get (GTK_TREE_MODEL (menu_tree), &child,
							    TITLE_COLUMN, &title,
							    -1);

					menu_str = g_strdup_printf ("%s,\"%s\"", menu_str, title);
					g_free (title);
		
					count++;
					valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (menu_tree), &child);
				}

				/* create the menu and pass it to the phone */

				menu_str = g_strdup_printf ("AT*EASM=%s,0,1,%d%s\r",
						            menu_title,
							    count,
							    menu_str);
				g_io_channel_write_chars (global.io_channel, menu_str, -1, NULL, NULL);
				g_io_channel_flush (global.io_channel, NULL);
			}
		}
		else if (depth > 1)
		{
			/* child menu called */
			i--;
			accessory_submenu (i, parentid);
		}
	}			
}

static void
accessory_submenu (gint child_id, gint parent_id)
{
	gchar *tmp, *command, *title;
	GtkTreeIter iter;
	
	/* get the right command */
	tmp = g_strdup_printf ("%d:%d", parent_id, child_id);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (menu_tree), &iter, tmp);
	gtk_tree_model_get (GTK_TREE_MODEL (menu_tree), &iter,
			    TITLE_COLUMN, &title,
			    COMMAND_COLUMN, &command,
			    -1);
	
	/* execute the command */
	g_spawn_command_line_async (command, NULL);
	
	/* free up the memory we've allocated */
	g_free (command);
	g_free (title);

	/* draw main menu */
	accessory_main_menu ();
}

static void
accessory_do_mouse ()
{
}
