/*  
    BluePhone is a Bluetooth mobile phone manager
    Copyright (C) 2004 Ed Rayne <er241@cam.ac.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/rfcomm.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>

#include <glib.h>
#include <gnome.h>

#include "global.h"
#include "bluetooth.h"
#include "phone.h"
#include "sms.h"
#include "gui.h"

static gboolean rfcomm_error (GIOChannel *source, GIOCondition *condition, gpointer data);
static gboolean rfcomm_hangup (GIOChannel *source, GIOCondition *condition, gpointer data);
static gboolean rfcomm_read (GIOChannel *source, GIOCondition *condition, gpointer data);

static gboolean 
rfcomm_error (GIOChannel *source, GIOCondition *condition, gpointer data)
{
	return FALSE;
}

static gboolean 
rfcomm_hangup (GIOChannel *source, GIOCondition *condition, gpointer data)
{
	g_free (last_command);
	gtk_timeout_add (TIMEOUT, (GtkFunction) rfcomm_connect, NULL);
	return FALSE;
}

static gboolean 
rfcomm_read (GIOChannel *source, GIOCondition *condition, gpointer data)
{
	gchar *buffer;

	g_io_channel_read_line (source, &buffer, NULL, NULL, NULL); 
	
	last_command = g_strdup_printf ("%s", buffer);
	
	parse_phone_output (buffer);
	
	g_free (buffer);
	
	return TRUE;
}

gboolean
rfcomm_connect (gpointer data)
{
	gchar *buffer;
	gint s;
	struct sockaddr_rc svr_addr;
	sms_t *newsms;

	global.socket = 0;
	
	svr_addr.rc_family = AF_BLUETOOTH;
	bacpy (&svr_addr.rc_bdaddr, &(global.bdaddr));
	svr_addr.rc_channel = global.rfcomm;

	/* Create a socket */
	if ((s = socket (AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM)) == -1)
		return TRUE; /* maintain timeout */
	
	/* Bind it to the RFCOMM device specified */
	if (!connect (s, (struct sockaddr *) &svr_addr, sizeof (svr_addr)))
	{
		global.socket = s;

		/* set up IO channel from socket */
		global.io_channel = g_io_channel_unix_new (global.socket);
		g_io_channel_set_encoding (global.io_channel, NULL, NULL);

		/* attach listeners to IO channel */
		g_io_add_watch (global.io_channel, G_IO_IN, (GIOFunc) rfcomm_read, NULL);
		g_io_add_watch (global.io_channel, G_IO_ERR, (GIOFunc) rfcomm_error, NULL);
		g_io_add_watch (global.io_channel, G_IO_HUP, (GIOFunc) rfcomm_hangup, NULL);
		last_command = g_malloc0 (10240 * sizeof (last_command));
		
		/* incoming caller ID */
		rfcomm_command_and_wait ("AT+CLIP=1\r", "OK");		
		
		/* event monitoring */
		rfcomm_command_and_wait ("AT+CMER=3,2,0,1,0\r", "OK");

		/* SMS reporting */
		rfcomm_command_and_wait ("AT+CNMI=3,1,2,0\r", "OK");

		/* add accessory menu */
		rfcomm_command_and_wait ("AT*EAM=\"BluePhone\"\r", "OK");
	
		/* connected successfully so disable the timeout */
		return FALSE;
	}
	else
	{
		/* failed to connect socket so maintain timeout */
		close (s);
		return TRUE;
	}
}

void
rfcomm_command (gchar *command)
{
	g_io_channel_write_chars (global.io_channel, command, -1, NULL, NULL);
	g_io_channel_flush (global.io_channel, NULL);
}

gchar *
rfcomm_command_and_wait (gchar *command, gchar *response)
{
	gchar *buffer;
	
	rfcomm_command (command);

	for (;;)
	{
		/* loop through any GTK events we need to stay up to date with */
		while (gtk_events_pending ())
			gtk_main_iteration ();

		if (strstr (last_command, response))
		{
			buffer = g_strdup_printf ("%s", last_command);
			break;
		}
		else if (strstr (last_command, "ERROR"))
		{
			buffer = NULL;
			break;
		}
	}
	
	/* just in case we get this far */
	last_command = g_strdup_printf ("");
	return buffer;
}

gchar *
rfcomm_get ()
{
	gchar *buffer = NULL;
	
	while (buffer == NULL)
	{
		if (gtk_events_pending ())
			gtk_main_iteration ();
		g_io_channel_read_line (global.io_channel, &buffer, NULL, NULL, NULL);
	}
	
	return buffer;
}

