/*  
    BluePhone is a Bluetooth mobile phone manager
    Copyright (C) 2004 Ed Rayne <er241@cam.ac.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <glib.h>

#include "gui.h"
#include "phone.h"
#include "bluetooth.h"
#include "sms.h"
#include "global.h"

static void parse_incoming_call (gchar *msg);
static void parse_incoming_sms (gchar *msg);
static void parse_keypress (gchar *msg);
static void parse_indicator_event (gchar *msg);

void
parse_phone_output (gchar *msg)
{
	if (g_strrstr (msg, "+CLIP:"))
		parse_incoming_call (msg);
	if (g_strrstr (msg, "+CMTI:"))
		parse_incoming_sms (msg);
	if (g_strrstr (msg, "+CKEV:"))
		parse_keypress (msg);
	if (g_strrstr (msg, "+CIEV:"))
		parse_indicator_event (msg);
	if (g_strrstr (msg, "*EAAI"))
		accessory_main_menu ();
	if (g_strrstr (msg, "*EAMI:"))
		accessory_menu_selected (msg);
}

static void
parse_incoming_call (gchar *msg)
{
	gchar *number;
	gint i;
	
	number = g_malloc0 (20 * sizeof (number));
	
	/* extract the number from the output */
	sscanf (msg, "+CLIP: \"%20[0-9]\"", number);

	/* bring up the dialog box */
	
	g_free (number);
}

static void
parse_incoming_sms (gchar *msg)
{
	sms_t *sms;
	gchar *number; 	/* sender's number */
	gchar *index;	/* index of recieved message */
	gchar *storage;	/* location of message ME or SM */
	gchar *pdu;
	gchar *command;
	gchar *buffer;

	/* need to put sms on the stack */
	sms = g_new (sms_t, 1);
	
	number = g_malloc0 (20 * sizeof (number));
	index = g_malloc0 (10 * sizeof (index));
	storage = g_malloc0 (10 * sizeof (storage));

	/* extract the details of the incoming message */
	sscanf (msg, "+CMTI: \"%2[A-Z]\",%5[0-9]", storage, index);

	command = g_strdup_printf ("AT+CPMS=\"%s\"\r", storage);
	rfcomm_command (command);
	for (;;)
	{
		buffer = rfcomm_get ();
		if (strstr (buffer, "OK"))
			break;
		if (strstr (buffer, "ERROR"))
			break;
	}
	command = g_strdup_printf ("AT+CMGR=%s\r", index);
	rfcomm_command (command);
	for (;;)
	{
		buffer = rfcomm_get ();
		if (strstr (buffer, "+CMGR:"))
			break;
		if (strstr (buffer, "ERROR"))
			break;
	}
	pdu = rfcomm_get ();
	
	sms = decode_sms_message (pdu);
	sms->messagestore = storage;
	sms->messageid = index;
	
	show_recievedsms_dialog (sms);
	
	g_free (number);
	g_free (index);
	g_free (storage);
	g_free (pdu);
	g_free (command);
	g_free (buffer);
}

static void
parse_keypress (gchar *msg)
{
	gchar a, b;

	a = *(msg+9);
	b = *(msg+11);
}

static void
parse_indicator_event (gchar *msg)
{
	gchar a, b;

	a = *(msg+9);
	b = *(msg+11);
}
